/* ------------------------------------------------------------ */
/* Conversion of proton data from HSCS-format to hevyold-format */
/* ------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dir.h>


#define LLEN  120  /* Maximal line length */

#define ERR(Z) { printf("\n\n ERROR : %s\n\n",Z); exit(1); }


/* Get A */

#define NEL 99  /* Number of elements in table */

double GetA(char *element)
{
      int i;

      static struct {   char *s;
                        double A;     } El[NEL] = {

      { "H"  ,   1.008 } , { "He" ,   4.003 } , { "Li" ,   6.941 } ,
      { "Be" ,   9.012 } , { "B"  ,  10.811 } , { "C"  ,  12.011 } ,
      { "N"  ,  14.007 } , { "O"  ,  15.999 } , { "F"  ,  18.998 } ,
      { "Ne" ,  20.180 } , { "Na" ,  22.990 } , { "Mg" ,  24.305 } ,
      { "Al" ,  26.982 } , { "Si" ,  28.086 } , { "P"  ,  30.974 } ,
      { "S"  ,  32.065 } , { "Cl" ,  35.453 } , { "Ar" ,  39.948 } ,
      { "K"  ,  39.098 } , { "Ca" ,  40.078 } , { "Sc" ,  44.956 } ,
      { "Ti" ,  47.867 } , { "V"  ,  50.942 } , { "Cr" ,  51.996 } ,
      { "Mn" ,  54.938 } , { "Fe" ,  55.845 } , { "Co" ,  58.933 } ,
      { "Ni" ,  58.693 } , { "Cu" ,  63.546 } , { "Zn" ,  65.409 } ,
      { "Ga" ,  69.723 } , { "Ge" ,  72.640 } , { "As" ,  74.922 } ,
      { "Se" ,  78.960 } , { "Br" ,  79.904 } , { "Kr" ,  83.798 } ,
      { "Rb" ,  85.468 } , { "Sr" ,  87.620 } , { "Y"  ,  88.906 } ,
      { "Zr" ,  91.224 } , { "Nb" ,  92.906 } , { "Mo" ,  95.940 } ,
      { "Tc" ,  98.000 } , { "Ru" , 101.070 } , { "Rh" , 102.906 } ,
      { "Pd" , 106.420 } , { "Ag" , 107.868 } , { "Cd" , 112.411 } ,
      { "In" , 114.818 } , { "Sn" , 118.710 } , { "Sb" , 121.760 } ,
      { "Te" , 127.600 } , { "I"  , 126.904 } , { "Xe" , 131.293 } ,
      { "Cs" , 132.905 } , { "Ba" , 137.327 } , { "La" , 138.906 } ,
      { "Ce" , 140.116 } , { "Pr" , 140.908 } , { "Nd" , 144.240 } ,
      { "Pm" , 145.000 } , { "Sm" , 150.360 } , { "Eu" , 151.964 } ,
      { "Gd" , 157.250 } , { "Tb" , 158.925 } , { "Dy" , 162.500 } ,
      { "Ho" , 164.930 } , { "Er" , 167.259 } , { "Tm" , 168.934 } ,
      { "Yb" , 173.040 } , { "Lu" , 174.967 } , { "Hf" , 178.490 } ,
      { "Ta" , 180.948 } , { "W"  , 183.840 } , { "Re" , 186.207 } ,
      { "Os" , 190.230 } , { "Ir" , 192.217 } , { "Pt" , 195.078 } ,
      { "Au" , 196.967 } , { "Hg" , 200.590 } , { "Tl" , 204.383 } ,
      { "Pb" , 207.200 } , { "Bi" , 208.980 } , { "Po" , 209.000 } ,
      { "At" , 210.000 } , { "Rn" , 222.000 } , { "Fr" , 223.000 } ,
      { "Ra" , 226.000 } , { "Ac" , 227.000 } , { "Th" , 232.038 } ,
      { "Pa" , 231.036 } , { "U"  , 238.029 } , { "Np" , 237.000 } ,
      { "Pu" , 244.000 } , { "Am" , 243.000 } , { "Cm" , 247.000 } ,
      { "Bk" , 247.000 } , { "Cf" , 251.000 } , { "E"  , 252.000 }

                                                  };


      for(i=0;i<NEL;++i)
        if(!strnicmp(element,El[i].s,strlen(El[i].s))) break;

      if(i>=NEL) return(-1.0);
            else return(El[i].A);

} /* End of GetA */


/* Copy substring [s1,s2] to 'to' */

void FromTo(char *source, int s1, int s2, char *target, int to)
{
      int i;

      for(i=s1;i<=s2;++i) target[to+i-s1]=source[i];

} /* End of FromTo */


/* Process file copy --> 1 if error */

int ProcessFile(char *target, char *in, char *on)
{

      FILE *fi,*fo;
      static char line[LLEN];
      double E,S;
      double A;


      /* Get A */

      if((A=GetA(target)) < 0.0) return(1);

      /* Open input+output file */

      if((fi=fopen(in,"rt"))==NULL) return(1);
      if((fo=fopen(on,"wt"))==NULL) ERR("output file open");

      /* Copy content */

      while(fgets(line,LLEN,fi)!=NULL)
      {
        sscanf(line+13,"%lf %lf",&E,&S);
        fprintf(fo,"%lE  %lE\n",E/1000.0,S/(1.66032*A));
      }

      /* Close files */

      if(fclose(fi)) ERR("input file close");
      if(fclose(fo)) ERR("output file close");

      return(0);

} /* End of ProcessFile */


main()
{
      FILE *hscs,*prot;
      static char line[LLEN], outl[LLEN];
      static char ipath[80], opath[80];
      static char target[30], filen[30];
      int i;


      /* Header */

      printf("\n PROGRAM JPCONV - VERSION 1.0\n");
      printf(" ============================\n\n");
      printf("   Input  list file : hscs.prn\n");
      printf("   Output list file : protold.prn\n\n");

      /* Open list files */

      if((hscs=fopen(   "hscs.prn","rt"))==NULL)
          ERR("HSCS file open error");
      if((prot=fopen("protold.prn","wt"))==NULL)
          ERR("Prot file open error");

      /* Print out file header */

      fputs("       A1 Pr        Tg File#  Energy/A  (keV)",prot);
      fputs("     Author    Error (%) Remark\n",prot);

      /* Make top level directory */

      mkdir("001H");

      /* Main I/O loop */

      while(fgets(line,LLEN,hscs) != NULL)
      {
        /* Initialize 'outl' */

        for(i=0;i<LLEN;++i) outl[i]=' ';
        outl[ 9]='1';
        outl[10]='H';

        /* Filename */

        for(i=88;i<=96;++i) if(line[i]=='.') break;
        FromTo(line,88,i-1,target,0);
        target[i-88]='\0';

        if(!isdigit(line[++i])) continue;
        for(;isdigit(line[i]) && i-88 < 20;++i);
        FromTo(line,88,i-1,filen,0);
        filen[i-88]='\0';

        FromTo(line,88,i-1,outl,21);

        /* Get i/o paths  & process file */

        strcpy(ipath,"01H\\");
        strcat(ipath,target);
        strcat(ipath,"\\");
        strcat(ipath,filen);

        strcpy(opath,"1H\\1H");
        strcat(opath,filen);

        if(ProcessFile(target,ipath,opath)) continue;

        printf("     FILE: %s\n",filen);

        /* Energy */

        FromTo(line,13,29,outl,13+18);

        /* Author */

        FromTo(line,44,48,outl,50);

        /* Error */

        for(i=33;i<40;++i) if(line[i]=='e') line[i]=' ';

        FromTo(line,33,39,outl,60);

        /* Add end of string mark  & print oline */

        outl[75]='\0';
        fprintf(prot,"%s\n",outl);

      }

      /* Close list files */

      if(fclose(hscs)) ERR("HSCS file close error");
      if(fclose(prot)) ERR("Prot file close error");

      return(0);

} /* End of main */

